import numpy


def prepare_data(new_data):
    import numpy
    from SEAScope.lib.utils import create_info
    # Compute range
    vmin = numpy.min(new_data)
    vmax = numpy.max(new_data)
    offset = vmin
    if vmin == vmax:
        scale = 1
    else:
        scale = (vmax - vmin) / 254
    fill_value = 255
    # Data must be flipped (up-down) and transposed for SEAScope
    # to use it properly
    result = numpy.flipud(new_data)
    result = result.transpose()
    # Pack data as uint8 (max value <=> 254, 255 is reserved for mask)
    data_packed = (result - offset) / scale
    data_clipped = numpy.clip(data_packed, 0, 254).astype('uint8')
    # Mask invalid values (255 is used as fill value)
    data_masked = numpy.ma.masked_invalid(data_clipped)
    data_masked[numpy.where(data_masked.mask)] = fill_value
    # Build description of data (shape, etc...)
    data_info = create_info(data_masked, 1, offset, scale, fill_value)
    # Data must be stored as a 1D array (required for serialization)       
    data_flattened = data_masked.flatten('F').tolist()
    return data_flattened, offset, scale, data_info

def prepare_metadata(metadata, data_info, data_flattened,
                     field_name,rid):
                     #granule_id, gcps, start, stop):
    # Fix an issue with the bindings (will not be required in the future)
    metadata['uris'][0]['shape']['xArity'] = int(metadata['uris'][0]['shape']['xArity'])
    metadata['uris'][0]['shape']['yArity'] = int(metadata['uris'][0]['shape']['yArity'])

    bucket = {"info": data_info,
              "buffer": data_flattened}
    granule = {"id": rid,
               "metadata": metadata,
               "data": {}}
    granule['data'][field_name] = bucket
    return granule

def add_data(data_info, data_flattened, field_name, granule):
     bucket = {"info": data_info,
               "buffer": data_flattened}
     granule['data'][field_name] = bucket
     return granule

def upload_granule(collection, granule):
    import SEAScope.upload

    # IP address and port used to reach the SEAScope standalone application
    #host = '192.34.177.4'
    host = '127.0.0.1'
    port = 11155

    with SEAScope.upload.connect(host, port) as link:
        # Send create collection job
        SEAScope.upload.collection(link, collection)

        # Send create granule job
        SEAScope.upload.granule(link, granule)

def upload_variable(variable):
    import SEAScope.upload

    # IP address and port used to reach the SEAScope standalone application
    #host = '192.34.177.4'
    host = '127.0.0.1'
    port = 11155

    with SEAScope.upload.connect(host, port) as link:
        # Send create variable job
        SEAScope.upload.variable(link, variable)

def upload_config(rcfg):
    import SEAScope.upload
    # IP address and port used to reach the SEAScope standalone application
    #host = '192.34.177.4'
    host = '127.0.0.1'
    port = 11155
  
    with SEAScope.upload.connect(host, port) as link:
        # Send create rendering config job
        SEAScope.upload.rendering_config(link, rcfg)
        
        
def init_ids(collection_offset, granule_offset):   
    import SEAScope.lib.utils
    _ = [next(SEAScope.lib.utils.collection_id_generator) for n in range(collection_offset)]
    _ = [next(SEAScope.lib.utils.granule_id_generator) for n in range(granule_offset)]